from abstract_consumer_handler import AbstractConsumerHandler





class MyHandler(AbstractConsumerHandler):
    
    def handle(self):
        print("handle")
        

class MyParentClass(MyHandler):
    pass
        



myHandler = MyParentClass()


class CustomError(Exception):
    """Base class for other exceptions"""
    pass

class TypeMissingError(CustomError):
    """Exception raised for errors in the subclass definition.

    Attributes:
        subclass_name -- name of the subclass which caused the error
        message -- explanation of the error
    """

    def __init__(self, subclass_name, message="Type attribute is missing in subclass"):
        self.subclass_name = subclass_name
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return f'{self.subclass_name} -> {self.message}'
    
    
class MethodMissingError(CustomError):
    """Exception raised for missing required methods in subclass.

    Attributes:
        subclass_name -- name of the subclass which caused the error
        method_name -- name of the missing method
        message -- explanation of the error
    """

    def __init__(self, subclass_name, method_name, message=None):
        self.subclass_name = subclass_name
        self.method_name = method_name
        if message is None:
            message = f"'{method_name}' method is missing in subclass '{subclass_name}'"
        self.message = message
        super().__init__(self.message)

    def __str__(self):
        return self.message    


from .abstract_consumer_handler_errors import TypeMissingError, MethodMissingError

HANDLER_ABSTRACT_CLASS_NAME= "AbstractConsumerHandler"


class HandlerMeta(type):
    def __new__(cls, name, bases, class_dict):
        new_cls = super().__new__(cls, name, bases, class_dict)
        if not hasattr(new_cls, 'handlers'):
            new_cls.handlers = {}
        is_handler_class_name_in_bases = HANDLER_ABSTRACT_CLASS_NAME in [base.__name__ for base in bases]            
        if not is_handler_class_name_in_bases:
            return new_cls
        if not hasattr(new_cls, 'type'):
            raise TypeMissingError(name)
        if not hasattr(class_dict, 'handle'):
            raise MethodMissingError(name, 'handle')
        new_cls.handlers[new_cls.type]=new_cls                        
        return new_cls                
        

class AbstractConsumerHandler(metaclass=HandlerMeta):
    '''
    Abstract class for websocket host handlers to subclass from
    Subclassed handlers should have type attribute and handler method    
    '''
    def __init__(self):
        super().__init__()


import unittest

from django_channels_utils.abstract_consumer_handler import AbstractConsumerHandler, MethodMissingError, TypeMissingError 

class TestAbstractConsumerHandler(unittest.TestCase):
    def test_subclass_missing_type(self):
        # Define a subclass without a type attribute
        with self.assertRaises(TypeMissingError):
            class MissingTypeHandler(AbstractConsumerHandler):
                def handle(self):
                    pass

    def test_subclass_missing_handle_method(self):
        # Define a subclass without a handle method
        with self.assertRaises(MethodMissingError):
            class MissingHandleMethodHandler(AbstractConsumerHandler):
                type = 'test'

    def test_subclass_with_all_requirements(self):
        # Define a valid subclass
        class ValidHandler(AbstractConsumerHandler):
            type = 'test'
            def handle(self):
                pass
        
        # Ensure that the handlers dictionary contains the new handler
        self.assertIn('test', AbstractConsumerHandler.handlers)
        self.assertEqual(AbstractConsumerHandler.handlers['test'], ValidHandler)

    def test_multiple_valid_subclasses(self):
        # Define multiple valid subclasses
        class FirstHandler(AbstractConsumerHandler):
            type = 'first'
            def handle(self):
                pass

        class SecondHandler(AbstractConsumerHandler):
            type = 'second'
            def handle(self):
                pass

        # Ensure that both handlers are registered correctly
        self.assertIn('first', AbstractConsumerHandler.handlers)
        self.assertEqual(AbstractConsumerHandler.handlers['first'], FirstHandler)

        self.assertIn('second', AbstractConsumerHandler.handlers)
        self.assertEqual(AbstractConsumerHandler.handlers['second'], SecondHandler)

    def test_handler_not_in_handlers_if_not_subclassed(self):
        # Define a class that is not a subclass of AbstractConsumerHandler
        class NonHandler:
            type = 'non_handler'
            def handle(self):
                pass

        # Ensure that the handlers dictionary does not contain this class
        self.assertNotIn('non_handler', AbstractConsumerHandler.handlers)


if __name__ == '__main__':
    unittest.main()
